#include <GL/glut.h>
#include <time.h>  

/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "GameObj.h"
#include "Bullet.h"
#include "Ship.h"
#include "GameSession.h"
#include "utility.h"

/** Global Variable storing the Game Session */
GameSession Game;

/** @name GLUT callbacks
@memo These functions are called back by GLUT main loop
	*/
//@{

/** Glut Callback on standard key input.
Mainly to get the space key shoot */
void Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
    exit(0);
  case ' ':
	 Game.Shoot();	 
  break;
  }
}

/** Glut Callback on pression of arrow keys.
It should start the thrust and the rotation of the ship.
*/
void  SpecialKey(int key, int x, int y)
{
  switch (key) {
  case GLUT_KEY_LEFT: 
	  Game.S->StartLeft();
	  break;
  case GLUT_KEY_RIGHT: 
	  Game.S->StartRight();
	  break;
  case GLUT_KEY_UP: 
	  Game.S->ThrustOn();
	  break;
	}
}
/** Glut Callback on release of arrow keys.
It should stop the thrust and the rotation of the ship.
*/
void SpecialKeyUp(int key, int x, int y)
{
  switch (key) {
  case GLUT_KEY_LEFT: 
	  Game.S->StopLeft();
	  break;
  case GLUT_KEY_RIGHT: 
	  Game.S->StopRight();
	  break;
  case GLUT_KEY_UP: 
	  Game.S->ThrustOff();
	  break;
	}
}

/** Opengl Initialization 
Standard thinghs for wirerame drawing
*/
void myGLInit(void)
{
  glDisable(GL_LIGHTING);
} 

/** Glut Callback on Window Reshape.
It updates the Projection Matrix and the viewport and stores the
new window size into memebers of GameSession object 
*/
void Reshape(GLsizei w, GLsizei h)
{ 
  Game.ScreenW=w;
  Game.ScreenH=h;
  int VL = -Game.SpaceSize;
  int VR =  Game.SpaceSize;
  int VB = -Game.SpaceSize*h/w;
  int VT =  Game.SpaceSize*h/w;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(VL,VR,VB,VT,-100,100);
  glMatrixMode(GL_MODELVIEW);
}		

/** Glut Idle Callback .
It is continuously called when events are not
being received. It post a refresh request in order to make the animation
as smoother as possible. It is enabled/disable by the Visible Callback
@see Visible
*/

void Idle(void)
{
	glutPostRedisplay();
}
/** Glut Callback on Window visibility.
It is called when the glut window change its visiblitiy status.
When the window is hidden we should stop the idle callback (and viceversa).
*/
void Visible(int vis)
{
 if (vis == GLUT_VISIBLE)
      glutIdleFunc(Idle);
 else
      glutIdleFunc(NULL);
}


/** Glut Callback to Draw everything.
It clear the screen, call the GameSession 
Update() and Draw() and swap the buffers.
Keep also track of the FPS rate.
*/
void Display(void)
{ 
  static int lasttime;
  int dt;
  dt=glutGet(GLUT_ELAPSED_TIME)-lasttime;
  lasttime=glutGet(GLUT_ELAPSED_TIME);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glPrintf(5,Game.ScreenH-20,20,"fps %3.1f\n",FPS(dt));
  glutGet(GLUT_ELAPSED_TIME);
  Game.Update(dt);
  Game.Draw();
  glutSwapBuffers();
}
//@}

/** The Classical Main Function
*/
int main(int argc, char** argv)
{ 
  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Asteroids");

  glutReshapeFunc(Reshape);
  glutDisplayFunc(Display);
  glutIdleFunc(Idle);
	glutVisibilityFunc(Visible);
  glutKeyboardFunc(Key);	
  glutSpecialFunc(SpecialKey);	
  glutSpecialUpFunc(SpecialKeyUp);	
    
  myGLInit(); 
	Game.Start();
  glutMainLoop();
  return 0;             /* ANSI C requires main to return int. */
}
